%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [var,ten1,ten2]=newton
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% density update for functional adaption
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% input:  F        -> deformation gradient
%         var      -> var = theta    ... internal variable growth factor
%         mat      -> material parameters
% output: var      -> var = theta    ... internal variable growth factor
%         ten1     -> tensor 1 for tangent operator    
%         ten2     -> tensor 2 for tangent operator    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tol = 1e-10;      
var =  0.0;
ndim = 2;
F = [1 0; 0.1 1.2];

emod = 1.0;
nue  = 0.3;
k0 = 1.0;
m  = 2.0;
t0 = 1.3;
dt = 1.0;

xmu = emod / 2.0 / (1.0+nue);
xlm = emod * nue / (1.0+nue) / ( 1.0-2.0*nue ); 
       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% euler backward - implicit time integration (quad.conv.@large str)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
the_k0 = var(1) + 1;
the_k1 = var(1) + 1;

iter = 0;   res  = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%% local newton-raphson iteration %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    while abs(res) > tol
      iter=iter+1;
  	  
	  Fe = F / the_k1;
	  Ce = Fe' * Fe;
	  I1 = trace(Ce);
	  Je = det(Ce);
	  delta  = eye(ndim); 
	  Fe_inv = inv(Fe);
	  Ce_inv = inv(Ce);
	  	  	  
	  Se = xmu * delta + (xlm/2 * log(Je) - xmu) * Ce_inv;
	  Pe = xmu * Fe    + (xlm/2 * log(Je) - xmu) * Fe_inv';  	  	  
	  Me = Ce*Se;
      
	  tr_Me = trace(Me);

      for i=1:ndim
      for j=1:ndim
      for k=1:ndim
      for l=1:ndim
          Le(i,j,k,l) =  xlm                    * Ce_inv(i,j)*Ce_inv(k,l) ...
			          - (xlm/2 * log(Je) - xmu) * Ce_inv(i,k)*Ce_inv(j,l) ...
			 		  - (xlm/2 * log(Je) - xmu) * Ce_inv(i,l)*Ce_inv(j,k);		 
      end, end, end, end      
      
      sca = 0;
      for i=1:ndim
      for j=1:ndim
      for k=1:ndim
      for l=1:ndim
          sca = sca + Ce(i,j) * Le(i,j,k,l) * Ce(k,l);		 
      end, end, end, end
	  
	  if the_k1==0
	    dtrM_dthe = 0;
	  else 
		dtrM_dthe = - 1/the_k1 * ( 2*tr_Me + sca );
	  end
	  
	  k       = k0*((t0-the_k1)/(t0-1))^m;
	  dk_dthe = k  /(the_k1-t0)        *m;
	  
	  dthe_dt = k * tr_Me; %%%%%%%%%%%%%%% ??? is it this simple??? %%%
	  
	  res  = dthe_dt * dt - the_k1 + the_k0;
	  dres =(dk_dthe * tr_Me + k * dtrM_dthe)*dt -1; 
	  
	  dthe =-res/dres;
	  the_k1 = the_k1 + dthe;
	  
%     res =((rho_k1/rho0)^(expn-expm)*psi0_neo-psi0)*dt-rho_k1+rho_k0; 
%	  dres= (expn-expm)*(rho_k1/rho0)^(expn-expm)*psi0_neo*dt/rho_k1-1;
	  
%     drho=- res/dres;  
%	  rho_k1 = rho_k1+drho;  
      
	  if(iter>20); disp(['*** NO LOCAL CONVERGENCE ***']); return; end;      
    end
%%% local newton-raphson iteration %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      
if (dres==0 | the_k1==1)
  fac = 0.0;

else
  fac = 2 / the_k1 / dres * k * dt;
end

for i=1:ndim
for j=1:ndim
    ten1(i,j) = 0;
	ten2(i,j) = Se(i,j);
for k=1:ndim
for l=1:ndim
	ten1(i,j) = ten1(i,j) + Le(i,j,k,l) * Ce(k,l);		 
	ten2(i,j) = ten2(i,j) + Le(i,j,k,l) * Ce(k,l)/2;		 
end, end, end, end

ten1 = fac * ten1;
% push & pull ten1 & ten2!

var(1) = the_k1 - 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
